# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

""" Schema test """

from cubicweb.devtools.testlib import CubicWebTC

class GenomicSchemaTC(CubicWebTC):
    """ Test proper behavior with respect to the composite relations. """
    def setup_database(self):
        """ Several entities involving composite relations are created,
            according to the schema.
        """
        req = self.request()
        t_gen_meas = req.create_entity('GenomicMeasure',
                                       type=u'Test measure type', format=u'raw')
        t_chrom = req.create_entity('Chromosome', name=u'Test chr 1', identifier=u'chr1')
        t_other_chrom = req.create_entity('Chromosome', name=u'Test chr 2', identifier=u'chr2')
        t_gen_reg = req.create_entity('GenomicRegion', start=5, stop=1000, width=500,
                                      cytoband_start=u'c123', cytoband_stop=u'c567',
                                      chromosome=t_chrom)
        t_gene = req.create_entity('Gene', gene_id=u'BRAF', chromosomes=[t_chrom, t_other_chrom])
        t_cgh_res = req.create_entity('CghResult', cgh_ratio=23.5e-4, log2_ratio=2.3e-5,
                                      status=u'S', numprobes=4, related_measure=t_gen_meas,
                                      genomic_region=t_gen_reg)
        t_mutation = req.create_entity('Mutation', position_in_gene=14,
                                       mutation_type=u'test type', reference_base=u'A',
                                       variant_base=u'G',
                                       related_measure=t_gen_meas,
                                       related_gene=t_gene)
        t_snp = req.create_entity('Snp', rs_id=u'test snp', position=123456789,
                                  chromosome=t_chrom, gene=t_gene)
        t_col_ref = req.create_entity('ColumnRef', name=u'red', type=u'Test colref type',
                                      measure=t_gen_meas)

    def test_cleanup_on_measure_delete(self):
        """ Test that on GenomicMeasure deletion, CghResult, Mutation and ColumnRef
            are deleted, while GenomicRegion, Gene and Chromosome are not deleted.
        """
        req = self.request()
        t_gen_meas = req.execute('Any X WHERE X is GenomicMeasure').get_entity(0, 0)
        req.execute('DELETE GenomicMeasure X WHERE X eid %(gmeseid)s',
                    {'gmeseid': t_gen_meas.eid})
        self.commit()
        db_cgh_res = req.execute('Any X WHERE X is CghResult')
        if db_cgh_res:
            self.fail('The CghResult was not deleted.')
        db_mutation = req.execute('Any X WHERE X is Mutation')
        if db_mutation:
            self.fail('The Mutation was not deleted.')
        db_colref = req.execute('Any X WHERE X is ColumnRef')
        if db_colref:
            self.fail('The ColumnRef was not deleted.')
        db_gen_reg = req.execute('Any X WHERE X is GenomicRegion')
        if not db_gen_reg:
            self.fail('The GenomicRegion was deleted.')
        db_gene = req.execute('Any X WHERE X is Gene')
        if not db_gene:
            self.fail('The Gene was deleted')
        db_chrom = req.execute('Any X WHERE X is Chromosome')
        if not db_chrom:
            self.fail('The Chromosome was deleted.')

    def test_cleanup_on_chromosome_delete(self):
        """ Test that on Chromosome deletion, GenomicRegion, CghResult
            and Snp are deleted, while Gene and GenomicMeasure are
            not deleted.
        """
        req = self.request()
        t_chrom = req.execute('Any X WHERE X is Chromosome, '
                              'X identifier "chr1"').get_entity(0, 0)
        req.execute('DELETE Chromosome X WHERE X eid %(chromeid)s',
                    {'chromeid': t_chrom.eid})
        self.commit()
        db_gen_reg = req.execute('Any X WHERE X is GenomicRegion')
        if db_gen_reg:
            self.fail('The GenomicRegion was not deleted.')
        db_cgh_res = req.execute('Any X WHERE X is CghResult')
        if db_cgh_res:
            self.fail('The CghResult was not deleted.')
        db_snp = req.execute('Any X WHERE X is Snp')
        if db_snp:
            self.fail('The Snp was not deleted.')
        db_gene = req.execute('Any X WHERE X is Gene')
        if not db_gene:
            self.fail('The Gene was deleted.')
        db_gen_meas = req.execute('Any X WHERE X is GenomicMeasure')
        if not db_gen_meas:
            self.fail('The GenomicMeasure was deleted.')

    def test_cleanup_on_region_delete(self):
        """ Test than on GenomicRegion delete, the CghResult
            is deleted, while no Chromosome is deleted.
        """
        req = self.request()
        t_gen_reg = req.execute('Any X WHERE X is GenomicRegion').get_entity(0, 0)
        req.execute('DELETE GenomicRegion X WHERE X eid %(gregeid)s',
                    {'gregeid': t_gen_reg.eid})
        self.commit()
        db_cgh_result = req.execute('Any X WHERE X is CghResult')
        if db_cgh_result:
            self.fail('The CghResult was not deleted.')
        db_chroms = req.execute('Any X WHERE X is Chromosome')
        if db_chroms.rowcount < 2:
            self.fail('Some Chromosomes have been deleted.')

    def test_cleanup_on_gene_delete(self):
        """ Test that on Gene deletion, Snp and Mutation are deleted,
            while no Chromosome is deleted.
        """
        req = self.request()
        t_gene = req.execute('Any X WHERE X is Gene').get_entity(0, 0)
        req.execute('DELETE Gene X WHERE X eid %(geid)s', {'geid': t_gene.eid})
        self.commit()
        db_snp = req.execute('Any X WHERE X is Snp')
        if db_snp:
            self.fail('The Snp was not deleted.')
        db_mutation = req.execute('Any X WHERE X is Mutation')
        if db_mutation:
            self.fail('The Mutation was not deleted.')


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from cubicweb.web import facet
from cubicweb.selectors import is_instance


############################################################################
### GENOMIC FACETS -CGH ####################################################
############################################################################
class CghResultCghRatioFacet(facet.RangeFacet):
    __regid__ = 'cghresult-cgh-ratio'
    __select__ = facet.RangeFacet.__select__ & is_instance('CghResult')
    order = 1
    rtype = 'cgh_ratio'


class CghResultLog2RatioFacet(facet.RangeFacet):
    __regid__ = 'cghresult-log2-ratio'
    __select__ = facet.RangeFacet.__select__ & is_instance('CghResult')
    order = 3
    rtype = 'log2_ratio'


class CghResultStatusFacet(facet.AttributeFacet):
    __regid__ = 'cghresult-status'
    __select__ = facet.AttributeFacet.__select__ & is_instance('CghResult')
    order = 3
    rtype = 'status'


class CghResulNumProbesFacet(facet.RangeFacet):
    __regid__ = 'cghresult-numprobes-ratio'
    __select__ = facet.RangeFacet.__select__ & is_instance('CghResult')
    order = 4
    rtype = 'numbprobes'


############################################################################
### GENOMIC FACETS -MUTATION ###############################################
############################################################################
class MutationPositionInGeneFacet(facet.RangeFacet):
    __regid__ = 'mutation-positioningene'
    __select__ = facet.RangeFacet.__select__ & is_instance('Mutation')
    order = 1
    rtype = 'position_in_gene'


class MutationMutationTypeFacet(facet.AttributeFacet):
    __regid__ = 'mutation-mutationtype'
    __select__ = facet.RangeFacet.__select__ & is_instance('Mutation')
    order = 2
    rtype = 'mutation_type'


class MutationPloidyFacet(facet.AttributeFacet):
    __regid__ = 'mutation-ploidy'
    __select__ = facet.AttributeFacet.__select__ & is_instance('Mutation')
    order = 3
    rtype = 'ploidy'


class MutationValidFacet(facet.AttributeFacet):
    __regid__ = 'mutation-valid'
    __select__ = facet.AttributeFacet.__select__ & is_instance('Mutation')
    order = 4
    rtype = 'valid'


class MutationGeneFacet(facet.RelationAttributeFacet):
    __regid__ = 'mutation-gene'
    __select__ = facet.RelationAttributeFacet.__select__ & is_instance('Mutation')
    rtype = 'related_gene'
    role = 'subject'
    order = 5
    target_attr = 'gene_id'


class MutationReferenceBaseFacet(facet.AttributeFacet):
    __regid__ = 'mutation-reference_base'
    __select__ = facet.AttributeFacet.__select__ & is_instance('Mutation')
    order = 6
    rtype = 'reference_base'


class MutationVariantBaseFacet(facet.AttributeFacet):
    __regid__ = 'mutation-variant_base'
    __select__ = facet.AttributeFacet.__select__ & is_instance('Mutation')
    order = 7
    rtype = 'variant_base'



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-genomics schema"""


from yams.buildobjs import (EntityType, RelationDefinition,
                            SubjectRelation, String, RichString,
                            BigInt, Int, Float, Boolean)


###############################################################################
### DATA / RESULTS ENTITIES ###################################################
###############################################################################
class CghResult(EntityType):
    """ CGH results """
    cgh_ratio = Float(required=True)
    log2_ratio = Float(required=True)
    status = String(required=True, maxsize=4)
    numprobes = Int(required=True)
    related_measure = SubjectRelation('GenomicMeasure', cardinality='?*', inlined=True, composite='object')
    genomic_region = SubjectRelation('GenomicRegion', cardinality='?*', inlined=True, composite='object')


class Mutation(EntityType):
    """ Mutational results """
    position_in_gene = Int(required=True) # Position
    mutation_type = String(required=True) # Type, first time
    mutation_id = String(maxsize=128) # Target ID
    ploidy = String(vocabulary=('Het', 'Hom'))
    valid = Boolean(indexed=True)
    biological_classification = String(maxsize=256) # Classification bio / annotation
    classification_variant = String(maxsize=256)
    dbsnp = String(maxsize=128)
    amplicon = String(maxsize=128)
    classification_type = String(maxsize=256) # Type, second time
    conclusions = String(maxsize=2048) # Concl
    related_measure = SubjectRelation('GenomicMeasure', cardinality='?*', inlined=True, composite='object')
    related_gene = SubjectRelation('Gene', cardinality='?*', inlined=True, composite='object')
    # Variant
    reference_base = String(maxsize=256, required=True) # Ref, first time
    variant_base = String(maxsize=256, required=True) # Variant
    variant_frequency = Float(indexed=True) # Var Freq
    p_value = Float() # P-value
    coverage = Float() # Coverage
    reference_coverage = Int() # Ref Cov
    variant_coverage = Int() # Var Cov
    # Ids
    hotspot_id = String(maxsize=256) # HotSpot
    reference_id = String(maxsize=256) # Ref, second time
    locus_version = String(maxsize=256) # NM
    # Additional infos - Fetched from external bases ?
    comment = String(maxsize=2048) # Comm.x
    protein = String(maxsize=256) # Prot
    nucl = String(maxsize=256) # Nucl
    polyphen = String(maxsize=256) # Polyphen
    sift = String(maxsize=256) # SIFT
    gvd_alignment = String(maxsize=256) # AlignGVD
    base = String(maxsize=256) # Base
    medical_coverage = Int() # Cov Med


###############################################################################
### GENETICS ENTITIES #########################################################
###############################################################################
class Chromosome(EntityType):
    """ Chromosome definition """
    name = String(required=True, unique=True, maxsize=16)
    identifier = String(required=True, indexed=True, maxsize=64)


class Gene(EntityType):
    """ Gene definition """
    name = String(maxsize=256, fulltextindexed=True, indexed=True)
    gene_id = String(maxsize=256, required=True, indexed=True)
    uri = String(maxsize=256, indexed=True)
    start_position = Int(indexed=True)
    stop_position = Int(indexed=True)
    # Allow for translocated genes
    chromosomes = SubjectRelation('Chromosome', cardinality='+*')


class GenomicRegion(EntityType):
    """ Genomic region definition """
    start = Int(required=True)
    stop = Int(required=True)
    width = Int(required=True)
    cytoband_start = String(required=True)
    cytoband_stop = String(required=True)
    chromosome = SubjectRelation('Chromosome', cardinality='1*', inlined=True, composite='object')
    genes = SubjectRelation('Gene', cardinality='**')
    oncogenes = SubjectRelation('Gene', cardinality='**')
    atlas_genes = SubjectRelation('Gene', cardinality='**')
    census_genes = SubjectRelation('Gene', cardinality='**')
    left_gene = SubjectRelation('Gene', cardinality='?*', inlined=True)
    right_gene = SubjectRelation('Gene', cardinality='?*', inlined=True)


class Snp(EntityType):
    """ SNP definition """
    rs_id = String(required=True, unique=True, maxsize=16)
    position = BigInt(required=True, indexed=True)
    chromosome = SubjectRelation('Chromosome', cardinality='1*', inlined=True, composite='object')
    gene = SubjectRelation('Gene', cardinality='**', composite='object')


###############################################################################
### MEASURE ENTITIES ##########################################################
###############################################################################
class GenomicMeasure(EntityType):
    """ A genomic measure """
    type = String(maxsize=256, required=True, indexed=True)
    format = String(maxsize=128, indexed=True)
    completed = Boolean(indexed=True)
    valid = Boolean(indexed=True)
    chip = SubjectRelation('Chip', cardinality='?*', inlined=True)
    column_on_chip = Int()
    row_on_chip = Int()
    conclusion = String(fulltextindexed=True)
    healthy_tissue = Boolean()
    mutated_genes = SubjectRelation('Gene', cardinality='**')


class GenomicPlatform(EntityType):
    name = String(required=True, maxsize=64)
    related_snps = SubjectRelation('Snp', cardinality='**')


class Chip(EntityType):
    barcode = Int(required=True, unique=True)
    platform = SubjectRelation('GenomicPlatform', cardinality='1*', inlined=True)


class ColumnRef(EntityType):
    """ Type is channel for transcriptom measure, lane for NGS measure.
        When type is channel, name will be red or green
    """
    name = String(vocabulary=('red', 'green'))
    type = String(maxsize=256, required=True, indexed=True)
    measure = SubjectRelation('GenomicMeasure', cardinality='1*', inlined=True, composite='object')



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-genomics entity's classes"""
import os.path as osp

from cubicweb.entities import AnyEntity, fetch_config


class GenomicMeasure(AnyEntity):
    __regid__ = 'GenomicMeasure'
    fetch_attrs, fetch_order = fetch_config(('type', 'format'))

    def dc_title(self):
        return u'%s (%s)' % (self._cw._('GenomicMeasure'), self.type)

    @property
    def image_url(self):
        return self._cw.data_url('48x48/genetics.png')

    @property
    def full_filepath(self):
        return self.filepath

    @property
    def subject_id(self):
        """ Return the subject id. Should be overloaded in concrete application
        of this cube """
        return None

    @property
    def formatted_chip_serialnum(self):
        """ Formatted chip serial num
        """
        if self.chip_serialnum:
            return self.chip_serialnum
        else:
            return 'N/A'


class GenomicRegion(AnyEntity):
    __regid__ = 'GenomicRegion'
    fetch_attrs, fetch_order = fetch_config(('start', 'stop', 'chromosome'))


    def dc_title(self):
        return '%s_%s: %s-%s' % (self._cw._('Chromosome'), self.chromosome[0].dc_title(),
                                 self.start, self.stop)


class Chip(AnyEntity):
    __regid__ = 'Chip'

    def dc_title(self):
        return '%s %s' % (self._cw._('Chip'), self.barcode)


class GenomicPlatform(AnyEntity):
    __regid__ = 'GenomicPlatform'

    def dc_title(self):
        return '%s %s' % (self._cw._('GenomicPlatform'), self.name)


class Chromosome(AnyEntity):
    __regid__ = 'Chromosome'

    def dc_title(self):
        return '%s %s' % (self._cw._('Chromosome'), self.name)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-genomics postcreate script, executed at instance creation time or when
the cube is added to an existing instance.

You could setup site properties or a workflow here for example.
"""

# Example of site property change
#set_property('ui.site-title', "<sitename>")

